import os
#from dotenv import load_dotenv
from .src.utils_aws_s3.s3_helper import AwsS3Helper
from .src.utils_local_os.local_helper import LocalFilesHelper
from .src.util_registry.util_registry import registry
import s3_bk_files.config as config
import coloredlogs, logging
import time
import datetime


logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG', logger=logger)
coloredlogs.install(level='success', logger=logger)

rg = registry()

aws_access_key_id       = os.getenv('ACCESS_KEY_ID')
aws_secret_access_key   = os.getenv('SECRET_ACCESS_KEY')
aws_region              = os.getenv('AWS_DEFAULT_REGION')

if aws_access_key_id and aws_secret_access_key and aws_region:
    logger.info('************** Leyendo Credenciales **************')
else:
    logger.error('No se ha configurado las variables de entorno.')
    exit(0)

#------------------------  MAIN -----------------------------------------
## Run function
def run():

    start_time = datetime.datetime.now()
    tic = time.time()
    
    try:
        s3Help = AwsS3Helper(aws_access_key_id,aws_secret_access_key,aws_region)
        # Obtener archivos del bucket ordenado
        # por defecto el primer elemento es el más antiguo
        lista_s3 = s3Help.sorted_list_keys(config.S3_BUCKET)
        # Obtener la fecha del ultimo archivo procesado
        last_datetime = rg.get_last_datetime_file_process()
        logger.info(' Fecha del último archivo procesado : '+str(last_datetime))

        ## Filtrar lista por la fecha anterior
        filter_list_s3 = s3Help.filter_by_datetime(lista_s3,last_datetime)
        if len(filter_list_s3) == 0 and last_datetime != None:
            logger.warning(' No hay nuevos archivos a procesar ... ')
        elif last_datetime == None:
            filter_list_s3 = lista_s3

        print('****  ')
        # Clase para gestionar carpetas locales
        locaTemplHelper = LocalFilesHelper(config.local_folder_temp)
        locaDefinitivelHelper = LocalFilesHelper(config.local_folder_definitive)


        # iterar archivos
        # guardar el último
        last_file = None
        for s3_file in filter_list_s3:
            local_dest_file = locaTemplHelper.create_pathfile_for_dict(s3_file)
            if local_dest_file:
                # Retorna True si ya esta registrado 
                is_registred = rg.registry_file_dict(s3_file)
                if not is_registred:
                    s3Help.download_file_from_dict(s3_file,local_dest_file)
                    print(local_dest_file)
                    # guardar en el otro directorio tambien
                    locaDefinitivelHelper.move_file(local_dest_file)
                    last_file = s3_file
            print('_'*60)

        toc = time.time()
        rg.save_process_time(start_time,tic,toc,last_file)

    except Exception as e:
        logger.error('Existe un problema en el proceso: %s', e)
    
    logger.info('************** Proceso Completado **************')

