import os
import json
import errno

import s3_bk_files.config as config
import coloredlogs, logging
logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG', logger=logger)


class registry:
    json_path = None
    data = None
    index = 0
    def __init__(self):
        if self.file_exists(config.json_registry_path):
            self.json_path = config.json_registry_path
            try:
                with open(self.json_path) as json_file:
                    self.data = json.load(json_file)
                self.index = len(self.data['history']) 
                self.data['history'].append({"process": {"files":[]}})
            except:
                with open(self.json_path) as json_file:
                    self.data = {"history":[{"process": {"files":[]}}]}
        else:
            # primera vez en caso no exista
            self.json_path = config.json_registry_path
            self.data = {"history":[{"process": {"files":[]}}]}
            self.save_data()
        

    def file_exists(self,path='registry.json'):
        return os.path.exists(path)

    def save_data(self):
        # Verficar si hay objetos en el nuevo proceso
        processed_files = len(self.data['history'][-1]['process']['files'])
        if processed_files == 0:
            return None
        try:
            with open(self.json_path, 'w') as f:
                json.dump(self.data, f, indent=4)
        except PermissionError as e:
            logger.error(e)
        except OSError  as e:
            if e.errno == errno.ENOSPC:
                logging.error(' No hay espacio suficiente en el disco.')
            else:
                logging.error(e)
        except Exception as e:
            logger.error(e)


    def get_last_datetime_file_process(self):
        if self.index == 0:
            return None
        
        last_datetime = self.data['history'][self.index-1]['process']['last_file_process']['info']['LastModified']
        return last_datetime
        


    def save_process(self,dict_process):
        '''
            self.data  =  { "history": [ process ...  ] }
        '''
        data = self.data
        data['history'].append(dict_process)
        self.save_data()

    
    def get_processed_files(self):
        '''
            self.data  =  { "history": [ "process": {  ... "files": [  files ...  ]   }  ] }
        '''
        data = self.data
        p_files = []
        
        try:
            for process in data['history']:
                for file in process['process']['files']:
                   p_files.append(file)
            return p_files
        except Exception as e:
            logger.warning(e)
            return []


    def find_file(self, filename):
        '''
            Busca archivo en el registro
            Return : None si no existe, el s3_file si existe

            get_processed_files =  [ {"name":" a.jpg"} ]

        '''
        ## Obtener archivos procesados
        procesed_files = self.get_processed_files()
        # buscar el archivo 
        rfile = None
        for file in procesed_files:
            if filename == file['name']:
                return file
        
        return rfile

    def is_processed(self,file_dict):
        s3_file = file_dict
        ## Obtener archivos procesados
        procesed_files = self.get_processed_files()
        # Verificar si el archivo nuevo coincide con el registrado
        flag = False
        for p_file in procesed_files:
            if p_file['pathS3'] == s3_file['S3path'] and p_file['datetime_modification'] == s3_file['info']['LastModified'].strftime(config.datetime_format_string):
                flag = True

        if flag:
            return True
        else:
            return False



    def get_name_from_dict(self,dict_file):
        if '/' in dict_file['S3path']:
            return dict_file['S3path'].split('/')[-1]

        
    
    def registry_file_dict(self,file_dict):
        """
            file_dict = {'S3path': 'my-bucket-sagemaker/OverallSystemUsage.json', 
            'info': {'Key': 'sagemaker/OverallSystemUsage.json',
             'LastModified': datetime.datetime(2021, 3, 29, 3, 36, 35, tzinfo=tzutc()), 
             'ETag': '"04a7284801e90eb4a69633a80b3610d0"', 
             'Size': 464, 'StorageClass': 'STANDARD', 
             'Owner': {'DisplayName': 'cquezada', 'ID': 'abb6d7141fbd42f63b986da5e37b7c6379a8cb7b156beea8c5270125cbe10ce0'}}}
        """
        # si ya ha sido procesado
        if self.is_processed(file_dict):
            # Comprobar si la fechas de modificacion coinciden
            
            #self.data['history'][self.index]['process']['files'].append(file)
            return True
        else:
            # Si no esta en el registro
            # Agregarlo
            file = self.formatDict(file_dict)
            print('file to save : ',file)
            self.data['history'][self.index]['process']['files'].append(file)
            self.save_data()
            return False



    def formatDict(self,file_dict):
        """
            INPUT = {'S3path': 'my-bucket-sagemaker/OverallSystemUsage.json', 
            'info': {'Key': 'sagemaker/OverallSystemUsage.json',
             'LastModified': datetime.datetime(2021, 3, 29, 3, 36, 35, tzinfo=tzutc()), 
             'ETag': '"04a7284801e90eb4a69633a80b3610d0"', 
             'Size': 464, 'StorageClass': 'STANDARD', 
             'Owner': {'DisplayName': 'cquezada', 'ID': 'abb6d7141fbd42f63b986da5e37b7c6379a8cb7b156beea8c5270125cbe10ce0'}}}

             OUTPUT:  {"name":"1.json","datetime_modification":"25-06-2021 15:32","pathS3":"1.json"}
        """
        name = self.get_name_from_dict(file_dict)
        datetime_modification = file_dict['info']['LastModified'].strftime(config.datetime_format_string)
        pathS3 = file_dict['S3path']
        
        return {"name":name,"datetime_modification":datetime_modification,"pathS3":pathS3}

    
    def save_process_time(self, start_time,tic,toc, last_file):
        mseconds = toc - tic
        self.data['history'][self.index]['process']['time_start'] = start_time.strftime(config.datetime_format_string)
        self.data['history'][self.index]['process']['seconds'] = str(toc-tic)
        self.data['history'][self.index]['process']['size'] = len(self.data['history'][self.index]['process']['files'])
        self.data['history'][self.index]['process']['bucket'] = config.S3_BUCKET
        ## Last index
        # format datetime
        if last_file:
            last_file['info']['LastModified'] = last_file['info']['LastModified'].strftime(config.datetime_format_string)
            self.data['history'][self.index]['process']['last_file_process'] = last_file

        self.save_data()


    # Obtener el último archivo por procesar

